@@ -12,6 +12,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <time.h>
 
 /**************************************************************************
  *  The following defines set the standard locations of sigrot's dir and 
@@ -61,6 +62,7 @@ int main(int argc, char *argv[])
 {
   FILE *inFile,*outFile;
   int next,test=0;
+  int total;
 
   /* Set the full path names */
   sprintf(Dest, "%s/%s", getenv("HOME"), DEST);
@@ -112,6 +114,18 @@ int main(int argc, char *argv[])
                 write_next(0);
                 printf("Signatures off.\n");
  	      }
+              if (!strcmp(argv[1], "-rand")) { /* get a random one */
+                    inFile=fopen(Source,"r");
+                    if (inFile==NULL)
+                       fprintf(stderr,"ERROR: Couldn't open \"%s\".\n", Source);
+                    else {
+                      total=count_all(inFile);
+                      next=(int) (time(0) % total);
+                      find_next_entry(&next,inFile);
+                      write_sig(inFile);
+                      fclose(inFile);
+                    }
+              }
               break;
       case 3: if (!strcmp(argv[1], "-w")) {
                 sprintf(buffer, "%s.bak", Source);
@@ -209,7 +223,7 @@ void write_next(int next) {
 void find_next_entry(int *next, FILE *inFile)
 {
   int localnext=*next;
-  char previous,current;
+  int previous,current;
 
   if (localnext!=1) {
     previous=getc(inFile);
@@ -226,6 +240,33 @@ void find_next_entry(int *next, FILE *inFile)
   }
 }
 
+/*******************************************************************
+ * count_all
+ *
+ * Takes as argument a file already open for input. It then searches
+ * for occurances of a double \n, or the end of file. It counts each
+ * of these occurances.
+ * It returns the number of entries in the file between double \n,
+ * and the end of file.
+ *******************************************************************/
+int count_all(FILE *inFile)
+{
+  int total = 0;
+  char previous, current;
+
+  previous=getc(inFile);
+  current=getc(inFile);
+  while(current != EOF) {
+    if ((current== '\n') && (previous == '\n')) ++total;
+    if ((current== EOF) && (previous != '\n')) ++total;
+    previous=current;
+    current=getc(inFile);
+  }
+  fseek(inFile,0L,0);
+  return total;
+}
+
+
 /*******************************************************************
  * write_sig
  * 
@@ -241,7 +282,7 @@ void write_sig(FILE *inFile)
 {
   FILE *outFile;
   int test=0;
-  char previous,current;
+  int previous,current;
 
   outFile=fopen(Dest,"w");
   if (outFile==NULL)
@@ -274,7 +315,7 @@ void write_sig(FILE *inFile)
 void copy_file (char *string,FILE *outFile)
 {
   FILE *inFile;
-  char tmp=0;
+  int tmp=0;
 
   inFile=fopen(string,"r");
   if (inFile!=NULL) {
